Emotional = LibStub("AceAddon-3.0"):NewAddon("Emotional", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
    char = {
		enableInParty = true,
		enableInRaid = true,
		enableDebug = false,
		emotes = {
				applaud = false,
				cackle = false,
				cheer = false,
				gloat = true,
				golfclap = false,
				guffaw = false,
				moo = false,
				mourn = false,
				rolleyes = false,
				sigh = false,
				train = false,
				violin = false,
		},
		available = {
				"gloat",
		}
		}
	}

local options = {
    name = "Emotional",
    handler = Emotional,
    type = 'group',
    args = {
		version = {
			order = 0,
			type = "description",
			name = "v3.0",
			width = "full",
		},
		desc = {
			order = 1,
			type = "description",
			name = "Automatic Emotion at Dead Raid/Party Members",
			width = "full",
		},
		emote = {
			order = -1,
			type = "multiselect",
			name = "Emotion",
			desc = "List of Available Emotions",
			get = "IsEnableEmotes",
			set = "ToggleEnableEmotes",
			width = "full",
			values = {
				["applaud"] = "You applaud at <target>. Bravo!",
				["cackle"] = "You cackle maniacally at <target>.",
				["cheer"] = "You cheer at <target>.",
				["gloat"] = "You gloat over <target>'s misfortune.",
				["golfclap"] = "You clap for <target>, clearly unimpressed.",
				["guffaw"] = "You take one look at <target> and let out a guffaw!",
				["moo"] = "You moo at <target>. Mooooooooooo.",
				["mourn"] = "In quiet contemplation, you mourn the death of <target>.",
				["rolleyes"] = "You roll your eyes at <target>.",
				["sigh"] = "You sigh a <target>.",
				["train"] = "Choo Choo!",
				["violin"] = "You plan the world's smallest violin for <target>.",
			}
		},
		enableInParty = {
			order = 5,
			type = "toggle",
			name = "Enable in Party",
			desc = "Enable/Disable in 5man Groups",
			get = "IsEnableInParty",
			set = "ToggleEnableInParty",
		},
		enableInRaid = {
			order = 6,
			type = "toggle",
			name = "Enable in Raid",
			desc = "Enable/Disable in Raid Groups",
			get = "IsEnableInRaid",
			set = "ToggleEnableInRaid",		
		},
		enableDebug = {
			order = 7,
			hidden = true,
			type = "toggle",
			name = "Debug",
			desc = "Enable Debug",
			get = "IsEnableDebug",
			set = "ToggleEnableDebug",
		},
    },
}

function Emotional:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EmotionalDB", defaults)
	
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Emotional", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Emotional", "Emotional")
end

function Emotional:DisplayStatus()
	print("|cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r")
	if self.db.char.enableInRaid then
		print("|cff58b058Enabled|r in Raid")
	else
		print("|cfff05032Disabled|r in Raid")
	end
	if self.db.char.enableInParty then
		print("|cff58b058Enabled|r in Party")
	else
		print("|cfff05032Disabled|r in Party")
	end
	if (self.db.char.enableDebug) then
		table.foreach(self.db.char.available, print)
	end
end

function Emotional:DisplayInfo()
	print("Welcome to |cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r Loaded")
	print(options.args.desc.name)
	print("Type |cff0078c1/emo|r to access configuration menu")
end

function Emotional:ChatCommand(input)
	if input == "debug" then
		if (self.db.char.enableDebug) then
			self.db.char.enableDebug = false
			print("Debug set to |cfff05032Disabled|r")
		else
			self.db.char.enableDebug = true
			print("Debug set to |cff58b058Enabled|r")
		end
	elseif (input == "test" or input == "run") and self.db.char.enableDebug then
			DoEmote(self:ChooseEmote(),"none")
	elseif input == "status" then
			self:DisplayStatus()
	elseif input == "info" or input == "help" then
			self:DisplayInfo()
	else
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	end
end

function Emotional:OnEnable()
	self:DisplayInfo()
	self:RegisterChatCommand("emo", "ChatCommand")
	self:RegisterChatCommand("emotional", "ChatCommand")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED","Action")
end

function Emotional:OnDisable()
	self:UnregisterChatCommand("emo", "ChatCommand")
	self:UnregisterChatCommand("emotional", "ChatCommand")
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Emotional:Action(...)
	if ((GetNumGroupMembers() > 5) and (self.db.char.enableInRaid)) or ((GetNumGroupMembers() < 6) and (GetNumGroupMembers() > 0) and (self.db.char.enableInParty)) then
		local arg0, arg1, event, arg3, arg4, caster, arg6, arg7, arg8, target, arg10, arg11, arg12, arg13 = select(1, ...);
		if (event == "UNIT_DIED") and (UnitIsDeadOrGhost(target)) and (UnitIsPlayer(target))then
			DoEmote(self:ChooseEmote(),target)
			if (self.db.char.enableDebug) then
				print(target, "has died.")
			end
		end
	end
end


function Emotional:IsEnableEmotes(info, key)
	return self.db.char.emotes[key]
end

function Emotional:ToggleEnableEmotes(info, key, newValue)
	if newValue then
		table.insert(self.db.char.available, key)
	else
		for i=1,table.getn(self.db.char.available),1 do
			if self.db.char.available[i] == key then
				table.remove(self.db.char.available, i)
				break
			end
		end
	end
	self.db.char.emotes[key] = newValue
end

function Emotional:ChooseEmote()
	return self.db.char.available[math.random(table.getn(self.db.char.available))]
end

function Emotional:IsEnableInParty(info)
	return self.db.char.enableInParty
end

function Emotional:ToggleEnableInParty(info, newValue)
	self.db.char.enableInParty = newValue
end

function Emotional:IsEnableInRaid(info)
	return self.db.char.enableInRaid
end

function Emotional:ToggleEnableInRaid(info, newValue)
	self.db.char.enableInRaid = newValue
end

function Emotional:IsEnableDebug(info)
	return self.db.char.enableDebug
end

function Emotional:ToggleEnableDebug(info, newValue)
	self.db.char.enableDebug = newValue
end